// Initial software: Apparent Relief Version 1.0
// Co-authors: Romain VERGNE, Pascal BARLA, Xavier GRANIER and Christophe SCHLICK.
// Owners: INRIA, University of Bordeaux 1 and University of Bordeaux 2.
// Copyright © 2009-2010, spread under the terms and conditions of the license CeCILL B Version 2.0.

#version 120

attribute float curvemin;
attribute float curvemax;

uniform float kratio; // curvature ratio

varying vec3  lightDir;
varying vec3  normal;
varying vec3  viewDir;
varying float kmin;
varying float kmax;
varying float depth;

const float tanhmax = 3.11622;

// S-Shape function
float tanh(float c) {
  float x = ((c*kratio)/tanhmax);
  float e = exp(-2.0*x);
  
  return clamp((1.0-e)/(1.0+e),-1.0,1.0);
}

void main(void) {
  vec3 vertex  = gl_Vertex.xyz;
  
  normal   = normalize(gl_NormalMatrix*gl_Normal);
  viewDir  = normalize((gl_ModelViewMatrix*gl_Vertex).xyz);
  lightDir = normalize(gl_LightSource[0].position.xyz-viewVertex);
  
  kmin     = tanh(min(curvemin,curvemax));
  kmax     = tanh(max(curvemin,curvemax));

  depth    = (gl_ModelViewMatrix*gl_Vertex).z;
  
  gl_Position  = ftransform();
}
