// Initial software: Apparent Relief Version 1.0
// Co-authors: Romain VERGNE, Pascal BARLA, Xavier GRANIER and Christophe SCHLICK.
// Owners: INRIA, University of Bordeaux 1 and University of Bordeaux 2.
// Copyright © 2009-2010, spread under the terms and conditions of the license CeCILL B Version 2.0.


#version 120
#extension GL_ARB_draw_buffers : enable

uniform sampler2D shapedata;
uniform float     width;     // screen width
uniform float     height;    // screen height

const float kernelSize   = 4;
const int   neighborhood = 9;

// gaussian derivative at order 0
float gd0[9] = float[9](0.00825977,0.0374983,0.110491,0.211309,0.262289,0.211309,0.110491,0.0374983,0.00825977);

// gaussian derivative at order 1
float gd1[9] = float[9](0.017613,0.0599706,0.117805,0.112648,-0,-0.112648,-0.117805,-0.0599706,-0.017613);

float Nx[9];
float Ny[9];
float K1[9];
float K2[9];

vec4  tmpGradient;
float tmpK1;
float tmpK2;

void loadDataAlongXAxis() {
  vec4  d;
  float x = -kernelSize;

  for(int i=0;i<neighborhood;i++) {
    vec2 c = vec2((gl_FragCoord.s+x)/width,gl_TexCoord[0].t);
    d     = texture2D(shapedata,c);
    Nx[i] = d.x;
    Ny[i] = d.y;
    K1[i] = d.z;
    K2[i] = d.w;

    x++;
  }

}

void convolveAlongXAxis() {
  tmpGradient = vec4(0.0);
  tmpK1       = 0.0;
  tmpK2       = 0.0;

  for(int i=0;i<neighborhood;i++) {
    
    // compute gradient for normal.x coordinate
    tmpGradient.x += Nx[i]*gd1[i];  // tmpGHx
    tmpGradient.y += Nx[i]*gd0[i];  // tmpGHy

    // compute gradient for normal.y coordinate
    tmpGradient.z += Ny[i]*gd1[i];  // tmpGVx
    tmpGradient.w += Ny[i]*gd0[i];  // tmpGVy
    
      // compute gaussian smoothing for K1
    tmpK1         += K1[i]*gd0[i];  // tmpK1
    
    // compute gaussian smoothing for K2
    tmpK2         += K2[i]*gd0[i];  // tmpK2
  }
}

void main(void)
{
  loadDataAlongXAxis();
  convolveAlongXAxis();
  
  // tmp gradient texture (texture name: tmpgradient1)
  gl_FragData[0] = tmpGradient;
  
  // nothing in tmp gradient texture 2 (used only with LODs effects)
  gl_FragData[1] = vec4(0.0);

  // tmp curvature texture (texture name: tmpcurvature)
  gl_FragData[2] = vec4(tmpK1,tmpK2,0.0,1.0);
}
