// Initial software: Apparent Relief Version 1.0
// Co-authors: Romain VERGNE, Pascal BARLA, Xavier GRANIER and Christophe SCHLICK.
// Owners: INRIA, University of Bordeaux 1 and University of Bordeaux 2.
// Copyright © 2009-2010, spread under the terms and conditions of the license CeCILL B Version 2.0.

#version 120
#extension GL_ARB_draw_buffers : enable

uniform sampler2D tmpgradient1;
uniform sampler2D tmpcurvature;
uniform float     width;  // screen width
uniform float     height; // screen height
uniform float     gratio; // gradient ratio

const float kernelSize   = 4;
const int   neighborhood = 9;
const float pi           = 3.14159265358;

// gaussian derivative at order 0
float gd0[9] = float[9](0.00825977,0.0374983,0.110491,0.211309,0.262289,0.211309,0.110491,0.0374983,0.00825977);

// gaussian derivative at order 1
float gd1[9] = float[9](0.017613,0.0599706,0.117805,0.112648,-0,-0.112648,-0.117805,-0.0599706,-0.017613);

float tmpGHx[9];
float tmpGHy[9];
float tmpGVx[9];
float tmpGVy[9];
float tmpK1 [9];
float tmpK2 [9];

vec2  GH;
vec2  GV;
float K1;
float K2;

// get directions along Y axis
void loadDataAlongYAxis() {
  vec4 d;
  vec2 k;
  vec2 c;
  float y = -kernelSize;

  for(int i=0;i<neighborhood;i++) {
    c         = vec2(gl_TexCoord[0].s,(gl_FragCoord.t-y)/height);
    d         = texture2D(tmpgradient1,c);
    k         = texture2D(tmpcurvature,c).xy;
    tmpGHx[i] = d.x;
    tmpGHy[i] = d.y;
    tmpGVx[i] = d.z;
    tmpGVy[i] = d.w;
    tmpK1 [i] = k.x;
    tmpK2 [i] = k.y;

    y++;
  }

}

// convolution along Y axis (Gradient differential structure)
void convolveAlongYAxis() {
  GH = GV = vec2(0.0);
  K1 = K2 = 0.0;

  for(int i=0;i<neighborhood;i++) {
    
    // compute gradient for normal.x coordinate
    GH.x += tmpGHx[i]*gd0[i];    // GHx
    GH.y += tmpGHy[i]*gd1[i];    // GHy

    // compute gradient for normal.y coordinate
    GV.x += tmpGVx[i]*gd0[i];    // GVx
    GV.y += tmpGVy[i]*gd1[i];    // GVy

    // compute gaussian smoothing for K1
    K1   += tmpK1[i]*gd0[i];     // K1
    
    // compute gaussian smoothing for K2
    K2   += tmpK2[i]*gd0[i];     // K2
  }
}


vec2 computeMultiImageGradient(in vec2 g1,in vec2 g2) {

/*   maximum gradient  */
/*   float x = abs(g1.x)>abs(g2.x) ? g1.x : g2.x; */
/*   float y = abs(g1.y)>abs(g2.y) ? g1.y : g2.y; */
/*   return vec2(x,y); */

/*   rms gradient  */
  float x = sqrt(g1.x*g1.x + g2.x*g2.x);
  float y = sqrt(g1.y*g1.y + g2.y*g2.y);
  return vec2(x,y);
}

float computeAmplitude(in vec2 g1,in vec2 g2) {
  float gxx = g1.x*g1.x + g2.x*g2.x;
  float gyy = g1.y*g1.y + g2.y*g2.y;
  float gxy = g1.x*g1.y + g2.x*g2.y;

  float theta1 = 0.5*atan(2.0*gxy/(gxx-gyy));
  float theta2 = theta1-0.5*pi;

  float amp1 = sqrt(abs(0.5*((gxx+gyy)+cos(2.0*theta1)*(gxx-gyy)+2.0*gxy*sin(2.0*theta1))));
  float amp2 = sqrt(abs(0.5*((gxx+gyy)+cos(2.0*theta2)*(gxx-gyy)+2.0*gxy*sin(2.0*theta2))));
  
  return max(amp1,amp2);
}

void main(void)
{
  loadDataAlongYAxis();
  convolveAlongYAxis();

  vec2 G = computeMultiImageGradient(GH,GV);
  float curvedness = computeAmplitude(GH,GV);

  // curvedness user control
  curvedness = (curvedness/(exp(gratio)*(1.0-curvedness)+curvedness));

  // gradient texture (texture name: gradient)
  gl_FragData[0] = vec4(G.x,G.y,curvedness,1.0);

  // curvature texture (texture name: curvature)
  gl_FragData[1] = vec4(K1,K2,0.0,1.0);
}
