// Initial software: Apparent Relief Version 1.0
// Co-authors: Romain VERGNE, Pascal BARLA, Xavier GRANIER and Christophe SCHLICK.
// Owners: INRIA, University of Bordeaux 1 and University of Bordeaux 2.
// Copyright © 2009-2010, spread under the terms and conditions of the license CeCILL B Version 2.0.

#version 120

uniform sampler2D utils;
uniform sampler2D gradient;
uniform sampler2D curvature;
uniform sampler2D arm;

vec2  shapeindex;
float curvedness;
float shapeindexlength;

void loadData() {
  // smoothed object space shape vector 
  shapeindex       = texture2D(curvature,gl_TexCoord[0].st).xy;
  
  // object space curvedness
  shapeindexlength = length(shapeindex);  
    
  // image space curvedness
  curvedness       = texture2D(gradient,gl_TexCoord[0].st).z;
}

vec2 shapeVector() {
  // combination of object and image space data
  float s    = smoothstep(0.0,0.1,shapeindexlength); // detect singular points
  curvedness = mix(shapeindexlength,curvedness,s);
  
  return shapeindexlength==0.0 ? vec2(0.0) : normalize(shapeindex)*curvedness;
}

vec4 getColorFromMap(in vec2 shapevector) {
  vec2 c = 0.5*(1.0+shapevector);
  
  return texture2D(arm,vec2(c.x,1.0-c.y));
}

float counterValue(in float relief,in float ndotl) {
  return (1.0-relief)*ndotl + relief*(1.0-ndotl);
}

void main() {
  loadData();

  float c = getColorFromMap(shapeVector()).x;         // relief value
  float l = 1.0-texture2D(utils,gl_TexCoord[0].st).x; // NdotL
  float d = counterValue(c,l);
  
  gl_FragColor = vec4(step(0.5,d));
}
